
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% Author: John-Olav Bratteb, 2014                      %
% Norwegian University of Science and Technology (NTNU) %
% jobrattebo@gmail.com                                  %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

close all;

%% Input
% startfile = 14069;    % Hele Case 1
% numberofbatches = 24; % Hele Case 1

% startfile = 14069;    % Simulering Case 1
% numberofbatches = 8;  % Simulering Case 1

% startfile = 18633;      % Case 2
% numberofbatches = 24;   % Case 2

startfile = 31121;    % Hele Case 3
numberofbatches = 58; % Hele Case 3

% startfile = 31121;    % testing
% numberofbatches = 2;  % testing

filesperbatch = 3;
batchstart = startfile:filesperbatch:startfile+numberofbatches*filesperbatch;

%% OUTPUT FILENAME 
savevideo = 0;      % OBS OBS check figurename (below) and filename in BirdFlightPath if turned on!
savefigures = 0;    % OBS OBS check figurename (below) and filename in BirdFlightPath if turned on!
savevariables = 0;  % OBS OBS check figurename (below) and filename in BirdFlightPath if turned on!

closefigure = 0;    % close figure from each batch

% figurename = 'Sim S1 Tracks CA6 CM6 MTI0 M4 DMHC1 DMUA1 SF1'; % Simuleringsdata Case 1
% figurename = 'Case1 Tracks CA10 MTI0 M4 DMHC1 DMUA1 SF1'; % Case 1
% figurename = 'Case2 Tracks CA6 CM6 MTI0 M4 DMHC1 DMUA1 SF1'; % Case 2
% figurename = 'Case3 Tracks CA6 CM6 MTI0 M4 DMHC1 DMUA1 SF1'; % Case 3
% figurename = '5perBatch Case1 Tracks CA6 CM6 MTI0 M3 DMHC1 DMUA1 SF1'; % Case 1   3 out of 5
figurename = '3perBatch Case3 Tracks CA8 CM8 MTI0 M2 DMHC1 DMUA1 SF1'; % Case 3   3 out of 5

%% Settings
max_misses_btwn_tracks = 2;
v_tol2 = 8;
a_tol2 = 30;

%% Preprocessing
if savevideo
    mkdir('Video',figurename); %#ok
end
if savevariables
    mkdir('Results',figurename);
end
imgmap = imread('Map_background2.png');
whitebg('black')
set(gcf,'Color','w')
set(gca,'Color','k')
set(gca,'XColor','k')
set(gca,'YColor','k')
set(gcf,'InvertHardcopy','off')

MatchedTracks = [];
% matchedpair1: [batch1#, track1#, batch2#, track2#;
% matchedpair2:  batch1#, track1#, batch2#, track2#; ...]

disp(' ')
%% Processing tracks 
for batch = 1:numberofbatches
    display(['Processing batch ', num2str(batch),'...'])
    files = batchstart(batch):batchstart(batch)+filesperbatch-1;
%     [Trackmatrix, Plotmatrix, Plotmatrix_c] = BirdFlightPath_v1_1_JOB(files);
    [bbs_norm, bbs_mti, Trackmatrix, Plotmatrix, Plotmatrix_c, s, svar, v, vvar, vr, vrd, vrdvar, tentative_tracks] = BirdFlightPath_v1_1_JOB(files);
    
%     Tidligere figur (4km vest):
%     imgmap = imread('Map_background.png');
%     trackfigure = figure();
%     p=Trackmatrix;
%     p = p(p>0);
%     scatter(Plotmatrix_c(p,1),Plotmatrix_c(p,2),10,Plotmatrix_c(p,4),'fill')
%     axis([-4000 4000 -3000 5000])
%     set(gcf,'Position',[300 -200 900 900])
%     set(gcf,'InvertHardcopy','off')
%     title([figurename, ' - Batch ',num2str(batch)])
%     hold on
%     imagesc([-4000 4000], [5000 -3000], imgmap)
%     alpha(0.45)
    
    trackfigure = figure();
    p=Trackmatrix;
    p = p(p>0);
    scatter(Plotmatrix_c(p,1),Plotmatrix_c(p,2),10,Plotmatrix_c(p,4),'fill')
    axis([-5000 5000 -5000 5000])
    set(gcf,'Position',[300 -200 900 900])
    set(gcf,'InvertHardcopy','off')
    title([figurename, ' - Batch ',num2str(batch)])
    hold on
    imagesc([-5000 5000], [5000 -5000], imgmap)
    alpha(0.5)
    caxis([0 255])
    
    Trackbearing = zeros(size(Trackmatrix,1),1);
    Trackspeed = zeros(size(Trackmatrix,1),1);
    for T = 1:size(Trackmatrix,1) % For all tracks T in current batch
        %% Calculate the bearing of each track in current batch
        % Track bearing is the bearing from the first to last plot in the track
        P = Trackmatrix(T,1);                                       % P = first plot in track
        Q = Trackmatrix(T,find(Trackmatrix(T,:)==0,1,'first')-1);   % Q = last plot in track
        if Trackmatrix(T,filesperbatch) ~= 0                                    % Special case if track contains plots in all scans of batch
            Q = Trackmatrix(T,filesperbatch);                                   
        end
        Px = Plotmatrix_c(P,1);
        Py = Plotmatrix_c(P,2);
        Qx = Plotmatrix_c(Q,1);
        Qy = Plotmatrix_c(Q,2);

        PQx = Qx - Px; % Track vector components
        PQy = Qy - Py;

        Trackbearing(T) = (180/pi)*asin(PQy/sqrt(PQx^2 + PQy^2)); % Bearing of track
        if PQx < 0                                          % Correction of bearing
            Trackbearing(T) = 180 - Trackbearing(T);
        elseif PQx > 0 && PQy < 0
            Trackbearing(T) = 360 - abs(Trackbearing(T));
        end
    
        %% Calculate the speed of each track
        % Track speed is the speed observed geometrically between the last two 
        % plots in the track
        Trackspeed(T) = v(T,find(v(T,:)==0,1,'first')-1);
    end
    
    if batch ~= 1 % Skip if in first batch
        %% Check for subsequent tracks corresponding to bird flight paths
        % NOTE: Track T is in previous batch, track i is in current batch
        disp(['Matching tracks from batch ' num2str(batch) ' to tracks of batch ' num2str(batch-1) '...'])
        for T = 1:size(Trackmatrix_prev,1) % For all tracks T in previous batch
            P = Trackmatrix_prev(T,find(Trackmatrix_prev(T,:)==0,1,'first')-2);   % P = 2nd last plot in track T in previous batch
            if Trackmatrix_prev(T,filesperbatch) ~= 0
                P = Trackmatrix_prev(T,filesperbatch-1);
            end
            Px = Plotmatrix_c_prev(P,1);
            Py = Plotmatrix_c_prev(P,2);
            Q = Trackmatrix_prev(T,find(Trackmatrix_prev(T,:)==0,1,'first')-1);   % Q = last plot in track T in previous batch
            if Trackmatrix_prev(T,filesperbatch) ~= 0
                Q = Trackmatrix_prev(T,filesperbatch);
            end
            Qx = Plotmatrix_c_prev(Q,1);
            Qy = Plotmatrix_c_prev(Q,2);

            dtPQ = Plotmatrix_c_prev(Q,3) - Plotmatrix_c_prev(P,3);       
            PQscandist = Plotmatrix_c_prev(Q,8) - Plotmatrix_c_prev(P,8);

            % Range Gate
            d_min = (Trackspeed_prev(T) - v_tol2)*dtPQ/PQscandist;
            d_max = (max_misses_btwn_tracks+1)*(Trackspeed_prev(T) + v_tol2)*dtPQ/PQscandist;

            % Search for tracks in extension of current track
            for i = 1:size(Trackmatrix,1) % For all tracks i (in current batch)
                S = Trackmatrix(i,1); % S = first plot of track i in current batch
                Sx = Plotmatrix_c(S,1); 
                Sy = Plotmatrix_c(S,2);

                dQS = sqrt( (Sx - Qx)^2 + (Sy - Qy)^2 ); % Distance from end of track T (prev batch) to start of track i (current batch)
                if d_min < dQS && dQS < d_max && Plotmatrix_c(S,8) - Plotmatrix_c_prev(Q,8) < max_misses_btwn_tracks % If S is within range gate, and not too many scans later
                    % Check difference in bearing in track T from
                    % previous batch to track i in current batch
                    QSx = Sx - Qx;
                    QSy = Sy - Qy;
                    QSbearing = (180/pi)*asin(QSy/sqrt(QSx^2 + QSy^2));
                    if QSx < 0
                        QSbearing = 180 - QSbearing;
                    elseif QSx > 0 && QSy < 0
                        QSbearing = 360 - abs(QSbearing);
                    end

                    if ismember(1,abs(Trackbearing_prev(T)-(QSbearing+[-360 0 360])) < a_tol2)  % If bearing to first plot S in track i is within Azimuth Gate
                         if ismember(1,abs(Trackbearing_prev(T)-(Trackbearing(i)+[-360 0 360])) < a_tol2) % If track i has less than a_tol difference in bearing from that of track T
                            % Track i of the current batch has been matched
                            % to track T of the previous batch!
                            MatchedTracks = [MatchedTracks; batch-1 T batch i];
                            
                            % Show track-matching in figure/video
                            % Line from/marker at first plot in track T
                            x1 = Plotmatrix_c_prev(Trackmatrix_prev(T,1),1);
                            y1 = Plotmatrix_c_prev(Trackmatrix_prev(T,1),2);
                            % Line to/marker at first plot in track i
                            x2 = Plotmatrix_c(Trackmatrix(i,1),1);
                            y2 = Plotmatrix_c(Trackmatrix(i,1),2);
                            % Plot line and markers
                            line([x1 x2], [y1 y2],'LineStyle','-','LineWidth',2,'Color',[1 0 0],'Marker','d','MarkerSize',10);
                         end
                    end

                end
            end
            
        end
    end
        
    %% Postprocessing
    % Keep the current trackmatrix, bearing, speed and plotmatrix for use in the next batch
    Trackmatrix_prev = Trackmatrix;
    Trackbearing_prev = Trackbearing;
    Trackspeed_prev = Trackspeed;
    Plotmatrix_c_prev = Plotmatrix_c;
    
    % Save variables
    if savevariables
        variablesname = ['Results/' figurename '/' figurename ' - Batch ' num2str(batch) '.mat'];
        save(variablesname, 'Trackmatrix', 'Plotmatrix', 'Plotmatrix_c', 'MatchedTracks', 's', 'svar', 'v', 'vvar', 'vr', 'vrd', 'vrdvar', 'tentative_tracks')
    end
    % Save figures
    if savefigures
        saveas(trackfigure, ['Figures/', figurename, ' - Batch ',num2str(batch),'.png'], 'png') %#ok
    end
    % Save figures for generation of video
    if savevideo
        saveas(trackfigure, ['Video/' figurename '/' num2str(batch) '.png'], 'png') %#ok
    end
    
    if closefigure
        close all
    end
end


%% Create video
if savevideo
    disp('Creating .mp4 video...')  %#ok
    % Construct a VideoWriter object
    outputVideo = VideoWriter(fullfile('Video', [figurename '.mp4']),'MPEG-4'); 
    outputVideo.FrameRate = 0.5;
    open(outputVideo);
    % Loop through the image sequence, load each image and write it to the video.
    for i = 1:numberofbatches
        filename = fullfile('Video',figurename,[num2str(i) '.png']);
        img = imread(filename);
        writeVideo(outputVideo,img);
    end
    % Finalize the video file.
    close(outputVideo);
    
    disp('Creating .avi video...')
    % Construct a VideoWriter object
    outputVideo = VideoWriter(fullfile('Video', [figurename '.avi'])); 
    outputVideo.FrameRate = 0.5;
    open(outputVideo);
    % Loop through the image sequence, load each image and write it to the video.
    for i = 1:numberofbatches
        filename = fullfile('Video',figurename,[num2str(i) '.png']);
        img = imread(filename);
        writeVideo(outputVideo,img);
    end
    % Finalize the video file.
    close(outputVideo);
end

disp('Finished!')

clearvars -except Plotmatrix_c Trackmatrix MatchedTracks bbs_mti bbs_norm