function [bbs_norm, bbs_mti, Trackmatrix, Plotmatrix, Plotmatrix_c, s, svar, v, vvar, vr, vrd, vrdvar, tentative_tracks] = BirdFlightPathDetector_v1_1_JOB(files)
% BirdFlightPathDetector_v1_1_JOB
%
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% Author: John-Olav Bratteb, 2014                      %
% Norwegian University of Science and Technology (NTNU) %
% jobrattebo@gmail.com                                  %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%
%{ 
%   VARIABLER:
% ------------------------------------------------------------------------
% - Plotmatrix bestr av en liste med alle plots fra alle scans.
% Hvor kolonne 1 er range i meter, 2 er asimut i grader, 3 er 
% observasjonstidspunkt, 4 er doppler-verdi, 5 er strrelse, 6 er range-celle,
% 7 er asimut-celle og 8 er scan nr.
%
% - Plotmatrix_c er samme som Plotmatrix, men der kolonne 1 er x og 2 er y,
% og koordinatene x og y er skalert om til at hver hele koordinat tilsvarer 
% 1 meter og vridd slik at orienteringen blir som i PPI, med nord rett opp.
%
% - Plotmatrix_cz er samme som Plotmatrix_c, men med en tredje dimensjon, der
% plottene er fordelt inn i 30 geografiske soner. Trenger ikke  brukes.
%
% - Trackmatrix bestr av en liste med plott, der hver rekke utgjr en Track.
% Trackmatrix(T,:) er track T. Denne rekken bestr av fra 2 til 7 plott
% sortert fra frst til sist observert (venstre til hyre), og zero-paddet
% til  ha size(Trackmatrix,2)=7.
%
% - v er en matrise bestende av de observerte hastighetene fra geometrien
% gitt av plott i Trackmatrix. Hastigheter sees kun i plott-par og gis ved
% avstand/(forskjell i observasjonstidspunkt). Derfor inneholder hver rekke
% i v ett mindre element enn samme rekke i Trackmatrix. [m/s]
%
% - vvar er en liste med variansen rundt gjennomsnittet til hastighetene 
% observert i et track (lagret i v). Variansen er normalisert mhp antallet 
% plott i tracket.
%
% - vr er radiell hastighet gitt av geometrisk observert hastighet.
% vr er som v, hvor hvert element er hastigheten mlt av geometrien
% mellom plott P og Q, men er her projisert p den radielle linjen gjennom
% origo og P. [m/s]
%
% - vrd er radiell hastighet gitt av doppler-verdien til den sist
% observerte av to plott. [m/s]
% 
% - s er en tabell som Trackmatrix, men der hvert element er strrelsen til 
% et plott i stedet for plottets nummer. [deteksjoner]
% 
% - svar er en liste med normalisert varianse av strrelsene til plott i
% hvert track, gitt i s. 
%
% v.1.1 - 15.juli 2014 - John-Olav Bratteb, NTNU
% Endrer til funksjon i stedet for script
%}

tic

%% Init
format long
global filename 
meters_per_range_cell = 1.25;
range_des_factor = 3;               % M settes lik det som settes i CNav
imgmap = imread('Map_background.png');
%% Figures
plotfigure = 0;
trackfigure = 0;
saveimage = 0;
trackstitle = 'CA6CM6-MTI0';
figurepos = [50 -50 800 800];
%% Tracking Parameters
N = 3;                  % Number of files per batch (M out of N criterion)
v_min = 10;             % Fuglenes antatte minste scan-til-scan hastighet [m/s]
v_max = 23;             % Fuglenes antatte strste scan-til-scan hastighet [m/s]
v_tol = 8;              % En fugls antatte strste forandring i hastighet over tre pflgende scans [m/s]
a_tol = 20;             % En fugls antatte strste forandring i retning fra scans[i,i+1] til scans[i+1,i+2] i grader
%% DMHC
DMHC = 1;               % Delete Measurements from High Clutter areas (DMHC)
DMUA = 1;               % Delete Measurements from Uninteresting Areas (DMUA)
DMHCthresh = 250;       % Terskel for hva som klassifiseres som High Clutter-omrder [0-255]
DMHCavg = 2;            % Cellestrrelse p spatial averaging filter.
% load('UAmap_case1.mat') % Laster inn kart over Uninteresting Areas (bilveier o.l.) for case1
% load('UAmap_case2.mat') % Laster inn kart over Uninteresting Areas (bilveier o.l.) for case2
load('UAgeomap2.mat')    % Laster inn kart over Uninteresting Areas (bilveier o.l.) som er fast for terrenget 360grader ut til 5.2km
%% Tolerances for track eval
Track_min_M = 2;        % M out of N criterion. Minimum antall plott som m til for  danne bekreftet track. Heltall mellom 2 og N

removebadtracks = 1;    % 0 eller 1. Hvis 1, still de 4 kravene under
% --------  --------  --------  --------
max_vvar = 40;          % Max tillatt normalisert varianse i geometrisk observert hastighet for et track
max_vrdvar = 50;        % Max tillatt normalisert varianse i dopplerverdier til plott i et track
max_size_var = 10000;   % Max tillatt normalisert varianse i strrelsen til plott i et track
dop_dev = 5;            % Terskel for avvik mellom en geometrisk radiell hastighet og en doppler-utledet hastighet [m/s]
dop_dev_N = 2;          % Max tillatte antall hastigheter med avvik strre enn dop_dev i et track
%% Generate Batch Plotmatrix
PRI = [];
for n = 1:size(files,2)   
%     filename = ['C:\Users\a\Documents\MATLAB\Masteroppgave\SimData\SimS1-CA6CM6-MTI0_260_220_1425_',num2str(files(n)),'.mat'];  % Simuleringsdata Case 1
%     filename = ['C:\Users\a\Documents\MATLAB\Masteroppgave\ProsessertVrnes\Pros-CA6CM6-MTI0_260_220_1425_',num2str(files(n)),'.mat'];  % Case 1
%     filename = ['C:\Users\a\Documents\MATLAB\Masteroppgave\TestPros\Pros-CA8CM10-MTI1_260_220_1425_',num2str(files(n)),'.mat'];  % Testing Case 1
%     filename = ['C:\Users\a\Documents\MATLAB\Masteroppgave\ProsessertVrnes\Pros-CA6CM6-MTI0_260_250_1425_',num2str(files(n)),'.mat'];  % Case 2
    filename = ['C:\Users\a\Documents\MATLAB\Masteroppgave\ProsessertVrnes\Case 3\Pros-CA8CM8-MTI0_260_250_1425_',num2str(files(n)),'.mat'];  % Case 3

    load(filename)
    if n == 1
        norm = zeros(size(bbs_norm,1),size(bbs_norm,2),size(files,2));  % Init av norm-map for estimering av HCareas
%         temp = GeneratePlotmatrix(double(bbs_mti));                 % Forenklet plott-ekstraktor   
        temp = GeneratePlotmatrix_TwoPass(double(bbs_mti));         % Two-Pass plott-ekstraktor
%         temp = GeneratePlotmatrix_TwoPass_des(double(bbs_mti));         % Two-Pass plott-ekstraktor med asimut-desimering
        Plotmatrix = [temp n*ones(size(temp,1),1)];                 % Legger til scan-stempel
        clear temp
    else
%         temp = GeneratePlotmatrix(double(bbs_mti));                 % Forenklet plott-ekstraktor
        temp = GeneratePlotmatrix_TwoPass(double(bbs_mti));         % Two-Pass plott-ekstraktor
%         temp = GeneratePlotmatrix_TwoPass_des(double(bbs_mti));         % Two-Pass plott-ekstraktor med asimut-desimering
        Plotmatrix = [Plotmatrix; temp n*ones(size(temp,1),1) ];    % Legger til scan-stempel
        clear temp
    end
    PRI = [PRI, 0.096e-6*(double(bbs_norm(3,2:end))*256+double(bbs_norm(4,2:end))) ];   % Lagrer alle PRI
    norm(:,:,n) = bbs_norm(:,:);                                         % Legger til nvrende bbs_norm i norm
end
PRF=round(1/mean(PRI));     % Beregner gjennomsnittlig PRF over alle filene i files
clear PRI;
Plotmatrix = double(Plotmatrix);
% toc % This part takes about 3.2 s

%% Generate, plot and save figure
% if plotfigure
%     Plotmatrixfigure = figure(); %#ok<UNRCH>
%     whitebg('black')                    % Bakgrunnsfarge inni figur
%     set(gcf,'Color','k')                % Bakgrunnsfarge i ramme
%     set(gca,'XColor','k')               % X-akse farge
%     set(gca,'YColor','k')               % Y-akse farge
%     set(get(gca,'Title'),'Color','k')   % Farge p figur-tittel
%     set(gcf,'InvertHardcopy','off')     % Lagre figur i farger slik vises (WYSIWYG)
%     set(gcf,'Position',[300 -50 900 900])
%     hold on
%     imagesc(bbs_norm)
%     alpha(0.2)
%     hold on
%     scatter(Plotmatrix(:,2),Plotmatrix(:,1),10,Plotmatrix(:,4),'fill')
%     if saveimage
%         saveas(Plotmatrixfigure, 'Plotmatrix.png','png');
%     end
% end

%% Limit total number of plots
% Dette er lite effektivt, og br fjernes/ikke brukes
%{
% if limitNNdist
%     % Plotmatrix_sorted = sortrows(Plotmatrix, [1,2]);
% 
%     % Removing all plots with nearest neighbor distance > max_NNdist
%     % tic
%     Plotmatrix2 = zeros(1,5);
%     for i = 1:size(Plotmatrix,1)
%         r1 = Plotmatrix(i,1)*range_des_factor*meters_per_range_cell;
%         th1 = Plotmatrix(i,2)*degrees_per_az_cell;
% 
%         for j = 1:size(Plotmatrix,1)
%             r2 = Plotmatrix(j,1)*range_des_factor*meters_per_range_cell;
%             th2 = Plotmatrix(j,2)*degrees_per_az_cell;
% 
%             if i~=j && Distance([r1,th1],[r2,th2],1) < max_NNdist
%                 Plotmatrix2 = [Plotmatrix2; Plotmatrix(i,:) ];
%                 break
%             end
%         end    
%     end
%     Plotmatrix2(1,:) = [];
%     clear r1 th1 r2 th2
%     % toc  % This part takes about 1.5 seconds
% end
%}

%% Delete Measurements from High Clutter areas (DMHC)
% and Delete Measurements from Uninteresting Areas (DMUA)

% Define High Clutter areas

    % for i = 2:size(files,2)
    %     norm(:,:,1) = norm(:,:,1) + norm(:,:,i);
    % end
    % norm = norm(:,:,1)/size(files,2);
    % norm(norm<DMHCthresh) = 0;
    % f1 = fspecial('average',5);
    % HCareas = imfilter(norm,f1,'replicate');
    % HCareas(HCareas>0)=1;

for i = 2:size(files,2)
    norm(:,:,1) = norm(:,:,1) + norm(:,:,i);
end
norm = norm(:,:,1)/size(files,2);
norm(norm<DMHCthresh) = 0;
f1 = fspecial('average',DMHCavg);
HCareas = imfilter(norm,f1,'replicate');
HCareas(HCareas>0)=1;
    
% Find measurements in HC and in Uninteresting areas
tobedeleted = [];
for p = 1:size(Plotmatrix,1)
    if ( DMHC && HCareas(Plotmatrix(p,6),Plotmatrix(p,7)) ) || ( DMUA && UAgeomap(floor(Plotmatrix(p,1)/11.25)+1,floor(Plotmatrix(p,2))+1) )  %UAmap(Plotmatrix(p,6),Plotmatrix(p,7)) )
        tobedeleted = [tobedeleted, p];
    end
end
% Delete measurements
Plotmatrix(tobedeleted,:) = [];
clear tobedeleted norm f1 UAgeomap

toc
%% Coordinate transform Polar (Plotmatrix 1 or 2) to Cartesian (Plotmatrix_c)
Plotmatrix_c = Plotmatrix;

% Vrir p koordinatsystem slik at det blir som i PPI
% og strekker p range-retningen slik at range-koordinaten blir oppgitt i meter
for i = 1:size(Plotmatrix)
    Plotmatrix_c(i,1) = (Plotmatrix(i,1))*cos(Plotmatrix(i,2)*pi/180); % x
    Plotmatrix_c(i,2) = (Plotmatrix(i,1))*sin(Plotmatrix(i,2)*pi/180); % y
end

% Plot Plotmatrix_c in cartesian coordinates
if plotfigure
    Plotsfigure = figure();
    whitebg('black')                    % Bakgrunnsfarge inni figur
    set(gcf,'Color','k')                % Bakgrunnsfarge i ramme
    set(gca,'XColor','k')               % X-akse farge
    set(gca,'YColor','k')               % Y-akse farge
    set(get(gca,'Title'),'Color','k')   % Farge p figur-tittel
    set(gcf,'InvertHardcopy','off')     % Lagre figur i farger slik vises (WYSIWYG)
    scatter(Plotmatrix_c(:,1),Plotmatrix_c(:,2),10,Plotmatrix_c(:,4),'fill')
    axis([-4000 4000 -3000 5000])
    set(gcf,'Position',figurepos)
    hold on
    imagesc([-4000 4000], [5000 -3000], imgmap)
    alpha(0.45)
    title('All Plots')
    caxis([0 255])
    if saveimage
        saveas(Plotsfigure, ['Plots',figurefilename], 'png')
    end
end
%% Divide Plotmatrix_c into geographical zones (Plotmatrix_cz)
%{
% Del opp Plots i grupper basert p hvor de er observert.
dec_lines_x = -5000:10000/6:5000;
dec_lines_y = -3000:8000/5:5000;
Plotmatrix_cz = zeros(size(Plotmatrix_c,1),size(Plotmatrix_c,2),(size(dec_lines_x,2)-1)*(size(dec_lines_y,2)-1));
for i = 1:size(Plotmatrix_c,1)
    br = 0;
    for j = 1:(size(dec_lines_y,2)-1)
        for k = 1:(size(dec_lines_x,2)-1)
            if Plotmatrix_c(i,1) > dec_lines_x(k) && Plotmatrix_c(i,1) < dec_lines_x(k+1) && Plotmatrix_c(i,2) > dec_lines_y(j) && Plotmatrix_c(i,2) < dec_lines_y(j+1)
                Plotmatrix_cz(find(Plotmatrix_cz(:,1,(j-1)*6+k)==0,1,'first'),:,(j-1)*6+k) = Plotmatrix_c(i,:);
                br = 1;
                break
            end
        end
        if br
            break
        end
    end
end % This takes about 0.04 s
clear br

% Strip trailing zeros
temp_max = 1;
for i = 1:size(Plotmatrix_cz,3)
    first_zero = find(Plotmatrix_cz(:,1,i)==0,1,'first');
    if  first_zero > temp_max
        temp_max = first_zero;
    end
end
Plotmatrix_cz = Plotmatrix_cz(1:temp_max,:,:);
clear temp_max first_zero

% Show Plots from specific geographical zone
% figure()
% scatter(Plotmatrix_cz(:,1,14),Plotmatrix_cz(:,2,14),10,Plotmatrix_cz(:,4,14),'fill')

%}
%% Find collinear plots
% Denne metoden ga for mange potensielle fuglebaner ut til  bli brukt
% videre.
% [Collinear_plots, Anglelist] = Find_collinear_plots(Plotmatrix_c,Ta);

% Collinear_plots = Find_collinear_plots(Plotmatrix_cz(1:find(Plotmatrix_cz(:,:,14)==0,1,'first'),:,14),Ta);

%% Find Birdtrack plots
% Finner starten p potensielle fuglebaner basert p avstanden mellom punkter.
% Sker deretter videre i de potensielle fuglebanene etter pllott som
% flger fuglebevegelser. 
% Returnerer Trackmatrisen som er en liste med sett av punkter 
% (i rekkeflgen de ble observert i) som utgjr potensielle fuglebaner.

[ Trackmatrix ] = Find_birdtracks(Plotmatrix_c, v_min, v_max, v_tol, a_tol, N);

% figure()
% p = Trackmatrix(Trackmatrix>0);
% scatter(Plotmatrix_c(p,1),Plotmatrix_c(p,2),10,Plotmatrix_c(p,4),'fill')
% axis([-4000 4000 -3000 5000])
% set(gcf,'Position',[300 -50 900 900])
% title('All tracks found')

%% Evaluation of tracks
% Fjern tracks med for f plott i track. Mange plott gir sterkere indikasjon p sanne ml.
Trackmatrix = Trackmatrix(Trackmatrix(:,Track_min_M)>0,:);

% Lagre antall tentative tracks
tentative_tracks = size(Trackmatrix,1);

% Evaluering av hastighetsdata
v = zeros(size(Trackmatrix));           % Hastighet fra geometri
vvar = Inf*ones(size(Trackmatrix,1),1); % Varians i mlt geometrisk hastighet i track
vr = zeros(size(v));                    % Radiell hastighet fra geometri
vrd = zeros(size(v));                   % Radiell hastighet fra dopplermling
vrdvar = zeros(size(Trackmatrix,1),1);  % Varians i hastighet fra doppler
for T = 1:size(Trackmatrix,1)
    for i = 1:size(Trackmatrix,2)-1
        if Trackmatrix(T,i+1)~=0
            P = Trackmatrix(T,i);
            Q = Trackmatrix(T,i+1);
            
            Px = Plotmatrix_c(P,1);
            Py = Plotmatrix_c(P,2);
            Qx = Plotmatrix_c(Q,1);
            Qy = Plotmatrix_c(Q,2);
            
            PQx = Qx - Px;
            PQy = Qy - Py;
            dPQ = sqrt( (Qx - Px)^2 + (Qy - Py)^2 );        % Avstand fra P til Q i meter
            dtPQ = Plotmatrix_c(Q,3) - Plotmatrix_c(P,3);   % Forskjell i obs.tidspkt fra P til Q mlt i sekunder
            
            v(T,i) = dPQ / dtPQ;                            % Geometrisk hastighet
            vr(T,i) = -v(T,i)*dot([PQx PQy],[Px Py]) / (sqrt(Px^2 + Py^2) * sqrt(PQx^2+PQy^2)); % Radiell hastighet fra geometri (positiv hastighet mot origo)
            
            % Pd = Plotmatrix_c(P,4);                         % Ps dopplerverdi
            Qd = Plotmatrix_c(Q,4);                         % Qs dopplerverdi
            
            vrd(T,i) = ((Qd-132)/256) * (3e8/9.41e9) * (PRF/2);  % Radiell hastighet fra dopplermling
        end
    end
    vvar(T) = var(v(T,v(T,:)>0),1);
    vrdvar(T) = var(vrd(T,v(T,:)>0),1);
end

% Evaluering av strrelser p deteksjonshopene i track
s = zeros(size(Trackmatrix));           % Strrelse p deteksjonshopene som dannet hvert plott
svar = Inf*ones(size(Trackmatrix,1),1); % Varians i strrelse p deteksjonshoper som dannet et track
for T = 1:size(Trackmatrix,1)
    for i = 1:size(Trackmatrix,2)
        if Trackmatrix(T,i) ~= 0
            s(T,i) = Plotmatrix(Trackmatrix(T,i),5);
        end
    end
    svar(T) = var(s(T,s(T,:)>0),1);
end

% Fjern tracks som er komplett gjengitt i andre tracks
% for i = 1:size(T,1)
%     for j = 1:size(T,1)
%        if i ~= j && all(ismember(T(i,T(i,:)>0),T(j,T(j,:)>0)))
%        T(i,:) = [];
%        break
%        end
%     end
% end
% OBS! Denne delen tar uhorvelig lang tid..

%% Remove bad tracks

% Vis tracks med Track_min_N eller flere plott
%{
Tracksfigure = figure();
p = Trackmatrix(Trackmatrix(:,Track_min_N)>0,:);
p = p(p>0);
scatter(Plotmatrix_c(p,1),Plotmatrix_c(p,2),10,Plotmatrix_c(p,4),'fill')
axis([-4000 4000 -3000 5000])
set(gcf,'Position',[300 50 1100 950])
title(['All tracks with ',num2str(Track_min_N),' or more plots'])
if saveimage
    saveas(Tracksfigure, ['Tracks',figurefilename], 'png')
end
%}

% Finn tracks som ikke mter kriteriene
if removebadtracks
    badtracks = []; %#ok<UNRCH>
    badtracks = [badtracks; find(vvar>max_vvar)];                   % Tracks hvor observert hastighet varierer mer enn max_vvar
    badtracks = [badtracks; find(vrdvar>max_vrdvar)];               % Tracks hvor dopplerverdi varierer mer enn max_vrdvar
    badtracks = [badtracks; find(sum(abs(vr-vrd)>dop_dev,2)>dop_dev_N)];    % Tracks hvor flere enn dop_dev_N hastigheter avviker mer enn dop_dev fra sine respektive doppler-mlte hastigheter
    badtracks = [badtracks; find(svar>max_size_var)];                       % Tracks hvor strrelsen til plott varierer mer enn max_size_var
    unique(badtracks);
    
    Trackmatrix(badtracks,:) = [];        % Remove bad tracks
    s(badtracks,:) = [];
    svar(badtracks,:) = [];
    v(badtracks,:) = [];
    vr(badtracks,:) = [];
    vrd(badtracks,:) = [];
    vrdvar(badtracks,:) = [];
    vvar(badtracks,:) = [];
end

%% Vis plots fra alle endelige tracks
if trackfigure
    trackfig=figure();
    p=Trackmatrix;
    p = p(p>0);
    scatter(Plotmatrix_c(p,1),Plotmatrix_c(p,2),10,Plotmatrix_c(p,4),'fill')
    axis([-4000 4000 -3000 5000])
    set(gcf,'Position',[300 -50 900 900])
    if removebadtracks
        figtitle = ['Final Tracks ' trackstitle ' M' num2str(Track_min_M) ' DMHC' num2str(DMHC) ' DMUA' num2str(DMUA)  ' SF1'];
        title(figtitle)
    else
        figtitle = ['Final Tracks ' trackstitle ' M' num2str(Track_min_M) ' DMHC' num2str(DMHC) ' DMUA' num2str(DMUA)  ' SF0'];
        title(figtitle)
    end
    whitebg('black')                    % Bakgrunnsfarge inni figur
    set(gcf,'Color','w')                % Bakgrunnsfarge i ramme
    set(gca,'XColor','k')               % X-akse farge
    set(gca,'YColor','k')               % Y-akse farge
    set(get(gca,'Title'),'Color','k')   % Farge p figur-tittel
    set(gcf,'InvertHardcopy','off')     % Lagre figur i farger slik vises (WYSIWYG)
    set(gcf,'Position',figurepos)
    hold on
    imagesc([-4000 4000], [5000 -3000], imgmap)
    alpha(0.45)
    caxis([0 255])
    if saveimage
        saveas(trackfig,[figtitle, '.png'],'png')
    end
end
%% Cleanup
clearvars -except Trackmatrix s svar v vvar vr vrd vrdvar Plotmatrix Plotmatrix_c bbs_norm bbs_mti HCareas tentative_tracks
toc

end