function det=CA_CFAR_v1_2(MTD_data,CFAR_win,CFAR_guard,threshold_mult,landclutt,landclutt_lvl,landclutt_mask_pros,landclutt_mask_near_lvl,landclutt_mask_far_lvl,calc_precision)
%{
Funksjon som bruker CA-CFAR til  detektere ml i et MTD prosessert 
basisbndsiggnal. 
Input:
MTD_data: Matrise med detekterte basisbnddata ut fra en puls doppler 
filter bank. Matrisa har dimensjon L x M x N, hvor L er antall
range-sampler, M er antall pulser , og N er antall dopplerfiltre som har
blitt benyttet.
CFAR_win: Strrelse p leading/trailing CFAR vindu. Skalar.
CFAR_guard: Antall guardceller p hver side av CUT.
threshold_mult: Threshold multiplier for CFAR. 
(Senere implementeres: Et threshold for hver
MTD-kanal, dvs threshold_mult er et array med N verdier)
landclutt: True/false, enabler deteksjon av landclutter i null-kanal med
fast terskel
landclutt_lvl: Terskel (liner verdi) for deteksjon av landclutter 
landclutt_mask_pros: True/false, enabler maskerning av sidelober fra landclutter
i null-kanalen
landclutt_mask_near_lvl: Spesifiserer sidelobeniv for landclutter mask. Skalar.
landclutt_mask_far_lvl: Spesifiserer sidelobeniv for landclutter mask. Skalar.
calc_precision: Beregningspresisjon (single eller double)
Output:
det: Matrise med deteksjoner (0,1)

Yngve Steinheim
SINTEF

v1.0 18 juli 2011

v1.1 30 sept 2011
1) Med dynamisk antall guardceller (input parameter)
2) Med fast terskel i null-kanalen som valgbar opsjon i input parametere

v1.2 8 nov 2011
1) Med sidelobe-maske i doppler for  unng deteksjoner p sidelober fra kraftig land-clutter.
2) Sletter variable i memory etter hvert som det ikke er bruk for dem


%}

%Test 190711
    % MTD_data(:,100,1)=0;
    % MTD_data(100,100,1)=10000;

CA_data=filter(ones(CFAR_win,1)/CFAR_win,1,MTD_data,[],1); %Beregner glidende mean

CA_data=CA_data(CFAR_win:end,:,:); %Fjerner CFAR_win-1 frste range sampler

size_rng_cells=size(MTD_data,1)-2*CFAR_win-2*CFAR_guard; %Output range (antall range celler som fr beregnet terskel)

det = zeros(size_rng_cells,size(MTD_data,2),size(MTD_data,3),calc_precision); %preallokering
%Test 081111
%     det = zeros(numb_cut,size(MTD_data,2),size(MTD_data,3),'int8'); 



CFAR_leading=CA_data(1:end-(2*CFAR_guard+1)-CFAR_win,:,:);
CFAR_trailing=CA_data(CFAR_win+(2*CFAR_guard+1)+1:end,:,:);
CFAR_backgr_est=(CFAR_leading+CFAR_trailing)/2; %Bakgrunnsestimat

clear CFAR_leading CFAR_trailing CA_data %Rydder i memory
 
MTD_data=MTD_data(CFAR_win+CFAR_guard+1:end-CFAR_win-CFAR_guard,:,:); %Alle CUT

%Frekvensmaske for kraftig clutter i null-kanalen

    if landclutt_mask_pros
        land_clutt_mask=zeros(size(MTD_data),calc_precision); %preallokering
        
        land_clutt_mask(:,:,1)=MTD_data(:,:,1)./landclutt_mask_far_lvl;
        
        for n=3:size(MTD_data,3)-1
            land_clutt_mask(:,:,n)=land_clutt_mask(:,:,1);
        end
        
        %Eget niv p nabo-kanaler til null-kanalen (frste
        %sidelober)
        land_clutt_mask(:,:,2)=MTD_data(:,:,1)./landclutt_mask_near_lvl;
        land_clutt_mask(:,:,size(MTD_data,3))=land_clutt_mask(:,:,2);
        
        CFAR_backgr_est(CFAR_backgr_est<land_clutt_mask)=land_clutt_mask(CFAR_backgr_est<land_clutt_mask);
    end
    
clear land_clutt_mask 
    
CFAR_thresh=CFAR_backgr_est.*threshold_mult;

clear CFAR_backgr_est

%Alternativ kortform (041011: Ikke lengre gyldig etter versjon 1.2):
%CFAR_thresh=((CA_data(1:end-3-CFAR_win,:,:)+CA_data(CFAR_win+4:end,:,:))/2).*threshold_mult;

%Fast lav terskel (istedet for CFAR)deteksjon i nullkanalen
    if landclutt
        CFAR_thresh(:,:,1)=landclutt_lvl;
    end

%Deteksjon p alle celler over CFAR threshold
    det(MTD_data>CFAR_thresh)=1;
    %Alternativt:
%         MTD_data(MTD_data<CFAR_thresh)=0; %Alle celler med niv under CFARterskel settes til null
%         det=MTD_data;                     %(det betyr at en beholder amplituden)

%Test 220711
    %det(:,:,1)=0; %Fjerner alle deteksjoner i 0-kanalen